#include <png.h>
#include <glib.h>
#include "png_util.h"

gint
canvas_export_png (Canvas      *canvas,
                   const gchar *path,
                   gint         x,
                   gint         y,
                   gint         width,
                   gint         height)
{
  CanvasRect *rect;
  guchar     *data;
  gint        result;

  data = g_malloc (width * 4 * height);
  if (!data)
    {
      return -1;
    }
  
  rect = canvas_rect_new (canvas, x, y, width, height);
  canvas_rect_export_buf (rect, data);
  g_object_unref (rect);
  result = horizon_save_png (path, width, height, width * 4, data);

  g_free (data);
  return result;
}

gint
canvas_import_png (Canvas      *canvas,
                   const gchar *path,
                   gint         x,
                   gint         y)
{
  CanvasRect *rect;
  gint        width;
  gint        height;
  gint        result;
  gint        row_stride;
  guchar     *src;

  result = horizon_load_png (path, &width, &height, &row_stride, &src);
  if (result || !src)
    {
      g_warning ("failed to open png file %s for reading", path);
      return -1;
    }

  rect = canvas_rect_new (canvas, x, y, width, height);
  canvas_rect_import_buf (rect, src);
  g_object_unref (rect);

  g_free (src);
  return result;
}

gint
horizon_query_png (const gchar *path,
                   gint        *width,
                   gint        *height)
{
  png_uint_32   w;
  png_uint_32   h;
  FILE         *infile;
  png_structp   load_png_ptr;
  png_infop     load_info_ptr;
  unsigned char header[8];

  png_bytep  *row_p = NULL;

  infile = fopen (path, "rb");
  if (!infile)
    {
      return -1;
    }

  fread (header, 1, 8, infile);
  if (png_sig_cmp (header, 0, 8))
    {
      fclose (infile);
      g_warning ("%s is not a png file", path);
      return -1;
    }

  load_png_ptr =
    png_create_read_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
  if (!load_png_ptr)
    {
      fclose (infile);
      return -1;
    }

  load_info_ptr = png_create_info_struct (load_png_ptr);
  if (!load_info_ptr)
    {
      png_destroy_read_struct (&load_png_ptr, &load_info_ptr, NULL);
      fclose (infile);
      return -1;
    }

  if (setjmp (png_jmpbuf (load_png_ptr)))
    {
      png_destroy_read_struct (&load_png_ptr, &load_info_ptr, NULL);
     if (row_p)
        g_free (row_p);
      fclose (infile);
      return -1;
    }

  png_init_io (load_png_ptr, infile);
  png_set_sig_bytes (load_png_ptr, 8);
  png_read_info (load_png_ptr, load_info_ptr);
  {
    int bit_depth;
    int color_type;

    png_get_IHDR (load_png_ptr, load_info_ptr, &w, &h, &bit_depth, &color_type, NULL, NULL, NULL);
    *width = w;
    *height = h;
    if ((bit_depth == 8) && (color_type == PNG_COLOR_TYPE_RGB))
      {
	png_set_add_alpha (load_png_ptr, 0xffffffff, 1);
      }
    else if (bit_depth != 8 || !(color_type == PNG_COLOR_TYPE_RGBA))
      {
        g_warning ("bpp or type mismatch");
        png_destroy_read_struct (&load_png_ptr, &load_info_ptr, NULL);
        fclose (infile);
        return -1;
      }
  }
  png_destroy_read_struct (&load_png_ptr, &load_info_ptr, NULL);
  return 0;
}

gint
horizon_load_png (const gchar  *path,
                  gint         *width,
                  gint         *height,
                  gint         *row_stride,
                  guchar      **pixels)
{
  png_uint_32   w;
  png_uint_32   h;
  FILE         *infile;
  png_structp   load_png_ptr;
  png_infop     load_info_ptr;
  unsigned char header[8];

  unsigned    int i;
  png_bytep  *row_p = NULL;

  infile = fopen (path, "rb");
  if (!infile)
    {
      return -1;
    }

  fread (header, 1, 8, infile);
  if (png_sig_cmp (header, 0, 8))
    {
      fclose (infile);
      g_warning ("%s is not a png file", path);
      return -1;
    }

  load_png_ptr =
    png_create_read_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
  if (!load_png_ptr)
    {
      fclose (infile);
      return -1;
    }

  load_info_ptr = png_create_info_struct (load_png_ptr);
  if (!load_info_ptr)
    {
      png_destroy_read_struct (&load_png_ptr, &load_info_ptr, NULL);
      fclose (infile);
      return -1;
    }

  if (setjmp (png_jmpbuf (load_png_ptr)))
    {
      png_destroy_read_struct (&load_png_ptr, &load_info_ptr, NULL);
     if (row_p)
        g_free (row_p);
      fclose (infile);
      return -1;
    }

  png_init_io (load_png_ptr, infile);
  png_set_sig_bytes (load_png_ptr, 8);
  png_read_info (load_png_ptr, load_info_ptr);
  {
    int bit_depth;
    int color_type;

    png_get_IHDR (load_png_ptr, load_info_ptr, &w, &h, &bit_depth, &color_type, NULL, NULL, NULL);
    *width = w;
    *height = h;
    if ((bit_depth == 8) && (color_type == PNG_COLOR_TYPE_RGB))
      {
	png_set_add_alpha (load_png_ptr, 0xffffffff, 1);
      }
    else if (bit_depth != 8 || !(color_type == PNG_COLOR_TYPE_RGBA))
      {
        g_warning ("bpp or type mismatch");
        png_destroy_read_struct (&load_png_ptr, &load_info_ptr, NULL);
        fclose (infile);
        return -1;
      }
    /*if (color_type == PNG_COLOR_TYPE_RGBA)
      png_set_strip_alpha (load_png_ptr);
     */  
  }

  *row_stride = w * 4;
  *pixels = g_malloc0 (*row_stride * h);

  row_p = (png_bytep *) g_malloc (sizeof (png_bytep) * h);

  for (i=0; i< h; i++)
    row_p[i] = *pixels + *row_stride * i;

  png_read_image (load_png_ptr, row_p);
  png_read_end (load_png_ptr, NULL);

  png_destroy_read_struct (&load_png_ptr, &load_info_ptr, NULL);

  if (row_p)
    g_free (row_p);
  fclose (infile);

  return 0;
}


gint
horizon_save_png (const gchar *path,
                  gint         width,
                  gint         height,
                  gint         row_stride,
                  guchar      *data)
{
  FILE          *fp;
  gint           i;
  png_struct    *png;
  png_info      *info;
  png_byte     **rows;
  png_color_16   white;

  fp = fopen (path, "wb");
  if (!fp)
    {
      return -1;
    }
  
  rows = g_malloc (height * sizeof (png_byte*));
  if (rows == NULL)
    return -1;

  for (i=0; i<height; i++)
    rows[i] = (png_byte *) data + i * row_stride;

  png = png_create_write_struct (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
  if (png == NULL)
    return -1;

  info = png_create_info_struct (png);

  if (setjmp (png_jmpbuf (png)))
    return -1;
  png_set_compression_level (png, 1);
  png_init_io (png, fp);

  png_set_IHDR (png, info,
     width, height, 8, PNG_COLOR_TYPE_RGB_ALPHA,
     PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_DEFAULT);

  white.red = 0xff;
  white.blue = 0xff;
  white.green = 0xff;
  png_set_bKGD (png, info, &white);

  png_write_info (png, info);
  png_write_image (png, rows);
  png_write_end (png, info);

  png_destroy_write_struct (&png, &info);
  g_free (rows);
  fclose (fp);

  return 0;
}
