#include <string.h>
#include <glib-object.h>
#include "gegl/gegl-types.h"
#include "gegl/gegl-pad.h"
#include "op_composer.h"
#include "canvas/types.h"
#include "canvas/canvas_rect.h"
#include "canvas/canvas.h"

enum
{
  PROP_0,
  PROP_OUTPUT,
  PROP_INPUT,
  PROP_AUX,
  PROP_IN_PLACE,
  PROP_LAST
};

static void  op_composer_class_init (OpComposerClass *klass);
static void  op_composer_init       (OpComposer      *self);

static void     finalize     (GObject      *gobject);
static void     get_property (GObject      *gobject,
                              guint         prop_id,
                              GValue       *value,
                              GParamSpec   *pspec);
static void     set_property (GObject      *gobject,
                              guint         prop_id,
                              const GValue *value,
                              GParamSpec   *pspec);
static gboolean evaluate     (GeglOperation   *operation,
                              const gchar  *output_prop);
static void     associate    (GeglOperation *operation);



G_DEFINE_TYPE (OpComposer, op_composer, GEGL_TYPE_OPERATION)


static void
op_composer_class_init (OpComposerClass * klass)
{
  GObjectClass    *object_class = G_OBJECT_CLASS (klass);
  GeglOperationClass *operation_class = GEGL_OPERATION_CLASS (klass);

  object_class->set_property = set_property;
  object_class->get_property = get_property;
  object_class->finalize     = finalize;

  operation_class->evaluate = evaluate;
  operation_class->associate = associate;

  g_object_class_install_property (object_class, PROP_OUTPUT,
                                   g_param_spec_object ("output",
                                                        "Output",
                                                        "An output property",
                                                        CANVAS_TYPE_RECT,
                                                        G_PARAM_READABLE |
                                                        GEGL_PAD_OUTPUT));

  g_object_class_install_property (object_class, PROP_INPUT,
                                   g_param_spec_object ("input",
                                                        "Input",
                                                        "An input property",
                                                        CANVAS_TYPE_RECT,
                                                        G_PARAM_CONSTRUCT |
                                                        G_PARAM_READWRITE |
                                                        GEGL_PAD_INPUT));

  g_object_class_install_property (object_class, PROP_AUX,
                                   g_param_spec_object ("aux",
                                                        "Input",
                                                        "Auxiliary image input",
                                                        CANVAS_TYPE_RECT,
                                                        G_PARAM_CONSTRUCT |
                                                        G_PARAM_READWRITE |
                                                        GEGL_PAD_INPUT));

  g_object_class_install_property (object_class, PROP_IN_PLACE,
                                   g_param_spec_boolean ("in-place",
                                                        "In place",
                                                        "do processing in-place, modifying the passed in drawable.",
                                                        FALSE,
                                                        G_PARAM_CONSTRUCT |
                                                        G_PARAM_READWRITE |
                                                        GEGL_PAD_INPUT));
}

static void
op_composer_init (OpComposer *self)
{
  self->in_place = FALSE;
  self->input    = NULL;
  self->aux      = NULL;
  self->output   = NULL;
}

static void
associate (GeglOperation *self)
{
  GeglOperation  *operation    = GEGL_OPERATION (self);
  GObjectClass   *object_class = G_OBJECT_GET_CLASS (self);

  gegl_operation_create_pad (operation,
                             g_object_class_find_property (object_class,
                                                           "output"));
  gegl_operation_create_pad (operation,
                             g_object_class_find_property (object_class,
                                                           "input"));
  gegl_operation_create_pad (operation,
                             g_object_class_find_property (object_class,
                                                           "aux"));
}

static void
finalize (GObject *object)
{
  OpComposer *self = OP_COMPOSER (object);

  if (self->output)
    g_object_unref (self->output);

  G_OBJECT_CLASS (op_composer_parent_class)->finalize (object);
}

static void
get_property (GObject      *object,
              guint         prop_id,
              GValue       *value,
              GParamSpec   *pspec)
{
  OpComposer *self = OP_COMPOSER (object);

  switch (prop_id)
  {
    case PROP_OUTPUT:
      g_value_set_object (value, self->output);
      break;
    case PROP_INPUT:
      g_value_set_object (value, self->input);
      break;
    case PROP_AUX:
      g_value_set_object (value, self->aux);
      break;
    case PROP_IN_PLACE:
      g_value_set_boolean (value, self->in_place);
      break;
    default:
      break;
  }
}

static void
set_property (GObject      *object,
              guint         prop_id,
              const GValue *value,
              GParamSpec   *pspec)
{
  OpComposer *self = OP_COMPOSER (object);

  switch (prop_id)
  {
    case PROP_INPUT:
      self->input = g_value_get_object(value);
      break;
    case PROP_AUX:
      self->aux = g_value_get_object(value);
      break;
    case PROP_IN_PLACE:
      self->in_place = g_value_get_boolean (value);
      break;
    default:
      break;
  }
}

static gboolean
evaluate (GeglOperation *operation,
          const gchar   *output_prop)
{
  OpComposer      *op_composer = OP_COMPOSER (operation);
  OpComposerClass *klass       = OP_COMPOSER_GET_CLASS (operation);
  gboolean success = FALSE;

  if (op_composer->input != NULL &&
      op_composer->aux   != NULL)
    {
      success = klass->evaluate (operation, output_prop);
    }
  else
    {
      g_warning ("%s received NULL %s%s%s",
          G_OBJECT_TYPE_NAME (operation),
          op_composer->input == NULL ? "input" : "",
          op_composer->input == NULL && op_composer->aux ? " and " : "",
          op_composer->aux ? "aux" : "");
    }
  if (op_composer->input)
    g_object_unref (op_composer->input);
  if (op_composer->aux)
    g_object_unref (op_composer->aux);

  return success;
}
